@@ -1,5 +1,5 @@
 /* 
- *    Copyright 2015, 2016 (c) Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
+ *    Copyright 2015, 2016, 2017, 2018 (c) Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
  * 
  *    file: font_ops.h
  *    This file is part of fontopia.
@@ -63,6 +63,12 @@ static inline UWORD swap_word(UWORD x)
                ((x<<8)&0xff00);    // move byte 0 to byte 1
 }
 
+static inline unsigned short swap_uword(unsigned short x)
+{
+	return ((x>>8)&0xff) |      // move byte 1 to byte 0
+               ((x<<8)&0xff00);    // move byte 0 to byte 1
+}
+
 
 enum file_state
 {
@@ -83,6 +89,7 @@ struct font_s
 #define VER_CP		0x3
 #define VER_RAW		0x4
 #define VER_BDF		0x5
+#define VER_PCF		0x6
 	unsigned char version;			/* PSF version */
 	unsigned int data_size;			/* total size of glyph data */
 	unsigned char *data;			/* pointer to glyph data */
@@ -102,20 +109,34 @@ struct font_s
 	unsigned int *unicode_table;		/* list of unicode entries of this font */
 	unsigned short *unicode_table_index;	/* index to unicode_table */
 	unsigned char utf_version;		/* format of unicode table: 1=UTF16, 2=UTF8 */
-	/* these fields are for use with Code Page (CP) files only. */
+	/* these fields are for use with Code Page (CP) and Windows FON files. */
 	char cp_active_font;
 	char cp_total_fonts;
 	/* which module is handling this font? */
 	struct module_s *module;
 	void *file_hdr;
 	unsigned int header_size;
-	char has_metadata;	/* flag to indicate if font has metadata (properties) e.g. like BDF & PCF */
+	char has_metadata;	/* flag to indicate if font has metadata (properties)
+                                 * e.g. like BDF & PCF
+                                 */
 	void *metadata;		/* pointer to metadata table if font has_metadata */
 	void *char_info;        /* pointer to auxiliary char info array (BDF only) */
 	unsigned int char_info_size;
 };
 
 
+struct char_info_s {
+    // for BDF fonts
+    int encoding;
+    int swidthX, swidthY;
+    int dwidthX, dwidthY;
+    int BBw, BBh, BBXoff, BBYoff;
+    // for PCF fonts
+    int rBearing, lBearing;
+    int charAscent, charDescent;
+};
+
+
 //void display_font_data(struct font_s *font, unsigned char *data);
 //void display_unicode_info(struct font_s *font);
 
@@ -134,5 +155,6 @@ struct font_s *create_empty_font();
 void force_font_dirty(struct font_s *font);
 int create_empty_unitab(struct font_s *font);
 int make_utf16(unsigned int *res, unsigned char *utf8);
+int create_char_info(struct font_s *font);
 
 #endif
