@@ -1,5 +1,5 @@
 /* 
- *    Copyright 2015, 2016 (c) Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
+ *    Copyright 2015, 2016, 2017, 2018 (c) Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
  * 
  *    file: glyph.h
  *    This file is part of fontopia.
@@ -21,6 +21,7 @@
 #ifndef GLYPH_H
 #define GLYPH_H
 
+/* glyph.c */
 void copy_glyph(struct font_s *font, unsigned char buffer[]);
 void cut_glyph(struct font_s *font, unsigned char buffer[]);
 void paste_glyph(struct font_s *font, unsigned char buffer[]);
@@ -33,4 +34,16 @@ void invert_glyph(struct font_s *font);
 int get_glyph_index(struct font_s *font);
 void draw_shape(struct font_s *font, char break_key);
 
+/* glyphext.c */
+void show_ext_glyph_operations(struct font_s *font);
+void glyphop_center_horz(struct font_s *font);
+void glyphop_center_vert(struct font_s *font);
+void glyphop_center_both(struct font_s *font);
+void glyphop_rotate_cw(struct font_s *font);
+void glyphop_rotate_ccw(struct font_s *font);
+
+/* glyphinfo.c */
+void show_glyph_info(struct font_s *font);
+int glyph_index(struct font_s *font);
+
 #endif
