@@ -1,5 +1,5 @@
 /* 
- *    Copyright 2015, 2016 (c) Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
+ *    Copyright 2015, 2016, 2017, 2018 (c) Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
  * 
  *    file: bdf.c
  *    This file is part of fontopia.
@@ -27,50 +27,50 @@
 struct bdf_keyword_s bdf_keywords[] =
 {
   /* Global keywords */
-  { "STARTFONT",       BDF_REQ_YES, BDF_GLOBAL_SCOPE, 1, NUMBER },
-  { "COMMENT",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, STRING },
-  { "CONTENTVERSION",  BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "FONT",            BDF_REQ_YES, BDF_GLOBAL_SCOPE, 1, STRING },
-  { "SIZE",            BDF_REQ_YES, BDF_GLOBAL_SCOPE, 3, NUMBER, NUMBER, NUMBER },
-  { "FONTBOUNDINGBOX", BDF_REQ_YES, BDF_GLOBAL_SCOPE, 4, INTEGER, INTEGER, INTEGER, INTEGER },
-  { "METRICSSET",      BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "SWIDTH",          BDF_REQ_NO,  BDF_ANY_SCOPE,    2, NUMBER, NUMBER },
-  { "DWIDTH",          BDF_REQ_NO,  BDF_ANY_SCOPE,    2, NUMBER, NUMBER },
-  { "SWIDTH1",         BDF_REQ_NO,  BDF_ANY_SCOPE,    2, NUMBER, NUMBER },
-  { "DWIDTH1",         BDF_REQ_NO,  BDF_ANY_SCOPE,    2, NUMBER, NUMBER },
-  { "VVECTOR",         BDF_REQ_NO,  BDF_ANY_SCOPE,    2, NUMBER, NUMBER },
-  { "STARTPROPERTIES", BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
+  { "STARTFONT",       BDF_REQ_YES, BDF_GLOBAL_SCOPE, 1, { NUMBER } },
+  { "COMMENT",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "CONTENTVERSION",  BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "FONT",            BDF_REQ_YES, BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "SIZE",            BDF_REQ_YES, BDF_GLOBAL_SCOPE, 3, { NUMBER, NUMBER, NUMBER } },
+  { "FONTBOUNDINGBOX", BDF_REQ_YES, BDF_GLOBAL_SCOPE, 4, { INTEGER, INTEGER, INTEGER, INTEGER } },
+  { "METRICSSET",      BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "SWIDTH",          BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
+  { "DWIDTH",          BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
+  { "SWIDTH1",         BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
+  { "DWIDTH1",         BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
+  { "VVECTOR",         BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
+  { "STARTPROPERTIES", BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
   { "ENDPROPERTIES",   BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 0 },
-  { "CHARS",           BDF_REQ_YES, BDF_GLOBAL_SCOPE, 1, INTEGER },
+  { "CHARS",           BDF_REQ_YES, BDF_GLOBAL_SCOPE, 1, { INTEGER } },
   { "ENDFONT",         BDF_REQ_YES, BDF_GLOBAL_SCOPE, 0 },
-  { "COPYRIGHT",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, STRING },
-  { "FOUNDRY",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, STRING },
-  { "FAMILY_NAME",     BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, STRING },
-  { "WEIGHT_NAME",     BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, STRING },
-  { "SLANT",           BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, STRING },
-  { "SETWIDTH_NAME",   BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, STRING },
-  { "FONT_VERSION",    BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, STRING },
-  { "FONT_TYPE",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, STRING },
-  { "PIXEL_SIZE",      BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "POINT_SIZE",      BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "RESOLUTION_X",    BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "RESOLUTION_Y",    BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "SPACING",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, STRING },
-  { "AVERAGE_WIDTH",   BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "CHARSET_REGISTRY",BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, STRING },
-  { "CHARSET_ENCODING",BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, STRING },
-  { "UNDERLINE_POSITION", BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "UNDERLINE_THICKNESS",BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "CAP_HEIGHT",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "X_HEIGHT",           BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "FONT_ASCENT",        BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "FONT_DESCENT",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "DEFAULT_CHAR",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, INTEGER },
-  { "ADD_STYLE_NAME",     BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, STRING },
+  { "COPYRIGHT",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "FOUNDRY",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "FAMILY_NAME",     BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "WEIGHT_NAME",     BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "SLANT",           BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "SETWIDTH_NAME",   BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "FONT_VERSION",    BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "FONT_TYPE",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "PIXEL_SIZE",      BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "POINT_SIZE",      BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "RESOLUTION_X",    BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "RESOLUTION_Y",    BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "SPACING",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "AVERAGE_WIDTH",   BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "CHARSET_REGISTRY",BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "CHARSET_ENCODING",BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "UNDERLINE_POSITION", BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "UNDERLINE_THICKNESS",BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "CAP_HEIGHT",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "X_HEIGHT",           BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "FONT_ASCENT",        BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "FONT_DESCENT",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "DEFAULT_CHAR",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "ADD_STYLE_NAME",     BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
   /* Glyph keywords */
-  { "STARTCHAR",       BDF_REQ_YES, BDF_GLYPH_SCOPE,  1, STRING },
-  { "ENCODING",        BDF_REQ_YES, BDF_GLYPH_SCOPE,  1, INTEGER },
-  { "BBX",             BDF_REQ_YES, BDF_GLYPH_SCOPE,  4, INTEGER, INTEGER, INTEGER, INTEGER },
+  { "STARTCHAR",       BDF_REQ_YES, BDF_GLYPH_SCOPE,  1, { STRING } },
+  { "ENCODING",        BDF_REQ_YES, BDF_GLYPH_SCOPE,  1, { INTEGER } },
+  { "BBX",             BDF_REQ_YES, BDF_GLYPH_SCOPE,  4, { INTEGER, INTEGER, INTEGER, INTEGER } },
   { "BITMAP",          BDF_REQ_YES, BDF_GLYPH_SCOPE,  0 },
   { "ENDCHAR",         BDF_REQ_YES, BDF_GLYPH_SCOPE,  0 },
 };
@@ -118,11 +118,11 @@ static inline char get_keyword_index(char *keyword)
     {
 	//printf("%s\n", keyword);
 	return i;
-	break;
     }
   }
   return i;
 }
+
 /* Helper function: get keyword argument of type STRING */
 static inline char *get_string(char **s)
 {
@@ -175,6 +175,7 @@ struct font_s *bdf_create_empty_font()
 	/* copy the standard metadata table, to fill it later */
 	memcpy(font->metadata, (void *)&metadata_table, sizeof(metadata_table));
 	//struct metadata_item_s *metadata = (struct metadata_item_s *)font->metadata;
+	if(!create_char_info(font)) goto memory_error;
 
 	font->length = 256;
 	font->has_unicode_table = 1;
@@ -299,7 +300,7 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 	/* check the version. we support ver 2.1 */
 	if(strcmp(s, "STARTFONT 2.1"))
 	{
-	  status_error("BDF version not supported");
+	  status_error("BDF versions apart from 2.1 are not supported");
 	  goto go;
 	}
 	char scope = BDF_GLOBAL_SCOPE;
@@ -420,10 +421,14 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 		font->data = data;
 		font->data_size = font->charsize*font->length;
                 font->unicode_info_size = (font->length * 4);
+                /*
                 char_info = (struct char_info_s *)malloc(font->length*sizeof(struct char_info_s));
                 if(!char_info) goto memory_error;
                 font->char_info = (void *)char_info;
                 font->char_info_size = font->length*sizeof(struct char_info_s);
+                */
+                if(!create_char_info(font)) goto memory_error;
+                char_info = font->char_info;
 	  }
 	  else if(index == STARTCHAR_KEYWORD)
 	  {
@@ -433,7 +438,6 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
                 unsigned int chr = 0;
 		if(s[0] != 'U' || s[1] != '+')
 		{
-		  //if(*s >= '2' && *s <= '7') chr = codepoint_to_unicode(s);
 		  if(*s >= '0' && *s <= '9') chr = get_hex(s, 4);
 		  else
                   {
@@ -444,31 +448,11 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
                           if(strlen(s) == 4) chr = get_hex(s, 4);
                       }
                   }
-                  /*
-                  if((*s >= 'A' && *s <= 'F') ||
-                          (*s >= 'a' && *s <= 'f'))
-                  {
-                      if(isspace(*(s+1)) || *(s+1) == '\0') chr = postscript_to_unicode(s);
-                      else 
-                      {
-                          if((*(s+1) >= '0' && *(s+1) <= '9') ||
-                              (*(s+1) >= 'A' && *(s+1) <= 'F') ||
-                              (*(s+1) >= 'a' && *(s+1) <= 'f'))
-                          {
-                              if(isspace(*(s+2)) || *(s+2) == '\0') chr = codepoint_to_unicode(s);
-                              else chr = postscript_to_unicode(s);
-                          }
-                          else chr = postscript_to_unicode(s);
-                      }
-                  }
-                  else if (isalpha(*s)) chr = postscript_to_unicode(s);
-                  */
                   if(!chr)
                   {
                       msgBox("Char name not in Unicode, Adobe\n"
                                     "Standard Encoding, or Postscript char\n"
                                     "name. I Can't read this encoding!", OK, ERROR);
-                      ////status_error("Char encoding not in Unicode. Can't read that!", OK, ERROR);
                       goto go;
                   }
 		}
@@ -569,14 +553,6 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 		    data[data_index+j] = 0;
                   }
 		  data_index += n;
-		  /*
-		  for(j = (1 << (w-1)); j >= 1; j >>= 1)
-		  {
-			if(line & j) putchar('X');
-			else putchar('.');
-		  }
-		  putchar('\n');
-		  */
 		  i--;
 		}
 		if(i) goto corrupt_file;
@@ -611,7 +587,7 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 	create_empty_unitab(font);
 	get_font_unicode_table(font);
 	calc_max_zoom(font);
-	font->state = NEW;
+	font->state = OPENED;
 	font->module = get_module_by_name("bdf");
 	/* something REALLY WRONG happended here */
 	if(!font->module) goto undefined_error;
@@ -620,7 +596,7 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 	return font;
 
 undefined_error:
-	status_error("Error creating new font");
+	status_error("Error loading font file");
 	//msgBox("Error creating new font", OK, ERROR);
 	goto go;
 corrupt_file:
@@ -736,21 +712,19 @@ int bdf_write_to_file(FILE *file, struct font_s *font)
         res = fprintf(file, "ENCODING %d\n", char_info[i].encoding);
         res = fprintf(file, "SWIDTH %d %d\n", char_info[i].swidthX, char_info[i].swidthY);
         res = fprintf(file, "DWIDTH %d %d\n", char_info[i].dwidthX, char_info[i].dwidthY);
-        /*
         res = fprintf(file, "BBX %d %d %d %d\n", char_info[i].BBw, char_info[i].BBh,
 					     char_info[i].BBXoff, char_info[i].BBYoff);
-        */
     }
     else
     {
         res = fprintf(file, "ENCODING %d\n", i);
         res = fprintf(file, "SWIDTH 1000 0\n");
         res = fprintf(file, "DWIDTH %d 0\n", font->width);
+        res = fprintf(file, "BBX %d %d %d %d\n", meta[METADATA_FONTBOUNDINGBOX_X].value,
+                                                 meta[METADATA_FONTBOUNDINGBOX_Y].value,
+                                                 meta[METADATA_FONTBOUNDINGBOX_XOFF].value,
+                                                 meta[METADATA_FONTBOUNDINGBOX_YOFF].value);
     }
-    res = fprintf(file, "BBX %d %d %d %d\n", meta[METADATA_FONTBOUNDINGBOX_X].value,
-					     meta[METADATA_FONTBOUNDINGBOX_Y].value,
-					     meta[METADATA_FONTBOUNDINGBOX_XOFF].value,
-					     meta[METADATA_FONTBOUNDINGBOX_YOFF].value);
     res = fprintf(file, "BITMAP\n");
     for(j = 0; j < font->height; j++)
     {
@@ -788,20 +762,33 @@ int bdf_write_to_file(FILE *file, struct font_s *font)
 
 void bdf_handle_hw_change(struct font_s *font, char *newdata, long new_datasize)
 {
-  //long old_datasize = font->data_size;
-  void *new_rawdata = (void *)malloc(new_datasize);
-  /* FIXME: Handle this error more decently */
-  if(!new_rawdata) return;
-  memcpy((void *)new_rawdata, (void *)newdata, new_datasize);
-  free(font->data);
-  font->data = new_rawdata;
-  font->data_size = new_datasize;
-  if(font->has_metadata)
-  {
-    struct metadata_item_s *meta = (struct metadata_item_s *)font->metadata;
-    meta[METADATA_FONTBOUNDINGBOX_X].value = font->width;
-    meta[METADATA_FONTBOUNDINGBOX_Y].value = font->height;
-  }
+    //long old_datasize = font->data_size;
+    void *new_rawdata = (void *)malloc(new_datasize);
+    /* FIXME: Handle this error more decently */
+    if(!new_rawdata) return;
+    memcpy((void *)new_rawdata, (void *)newdata, new_datasize);
+    free(font->data);
+    font->data = new_rawdata;
+    font->data_size = new_datasize;
+    if(font->has_metadata)
+    {
+        struct metadata_item_s *meta = (struct metadata_item_s *)font->metadata;
+        meta[METADATA_FONTBOUNDINGBOX_X].value = font->width;
+        meta[METADATA_FONTBOUNDINGBOX_Y].value = font->height;
+    }
+    if(font->char_info)
+    {
+        struct char_info_s *char_info = font->char_info;
+        int i;
+        for(i = 0; i < font->length; i++)
+        {
+            char_info[i].dwidthX     = font->width;
+            char_info[i].lBearing    = 0;
+            char_info[i].rBearing    = font->width;
+            char_info[i].charAscent  = font->height;
+            char_info[i].charDescent = 0;
+        }
+    }
 }
 
 /*
@@ -876,6 +863,10 @@ void bdf_handle_version_change(struct font_s *font, char old_version)
 	/* copy the standard metadata table, to fill it later */
 	memcpy(font->metadata, (void *)&metadata_table, sizeof(metadata_table));
   }
+  if(!font->char_info)
+  {
+      create_char_info(font);
+  }
   struct metadata_item_s *metadata = (struct metadata_item_s *)font->metadata;
   metadata[METADATA_FONTBOUNDINGBOX_X].value = font->width;
   metadata[METADATA_FONTBOUNDINGBOX_Y].value = font->height;
