@@ -1,5 +1,5 @@
 /* 
- *    Copyright 2015, 2016 (c) Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
+ *    Copyright 2015, 2016, 2017, 2018 (c) Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
  * 
  *    file: psf.c
  *    This file is part of fontopia.
@@ -431,20 +431,16 @@ struct font_s *psf_load_font(unsigned char *file_data, long file_size)
 	return font;
 
 undefined_error:
-	status_error("Error creating new font");
-	//msgBox("Error creating new font", OK, ERROR);
+	status_error("Error loading font file");
 	goto end;
 file_corrupt:
 	status_error("Font file is corrput!");
-	//msgBox("Font file is corrput!", OK, ERROR);
 	goto end;
 file_read_error:
 	status_error("Error reading font file");
-	//msgBox("Error reading font file", OK, ERROR);
 	goto end;
 memory_error:
 	status_error("Not enough memory to load font file");
-	//msgBox("Insufficient memory", OK, ERROR);
 end:
 	kill_font(font);
 	return (struct font_s *)NULL;
